#include <stdio.h>
#include <string.h>
#include <map>

using namespace std;

#define MAXLEVELS 51
#define NMAX 100111
#define MAXDIVISORS 222
#define MOD 1000000007
#define DEBUG 0

int gooddiv[NMAX][MAXDIVISORS], ngooddiv[NMAX], cnt[MAXLEVELS][NMAX], lcnt[MAXLEVELS][NMAX], ncnt[MAXLEVELS], touched[MAXLEVELS][NMAX], touch_idx = 0;
int v[NMAX], vlist[MAXLEVELS][NMAX], vmax[MAXLEVELS], vsize[MAXLEVELS], vsz[MAXLEVELS], base[MAXLEVELS], tcnt[MAXLEVELS][NMAX];
int bit[NMAX], fact[NMAX], factinv[NMAX], inv[NMAX], comb[NMAX], nprime[NMAX], bad[NMAX];
int N, K, answer1, answer2, a1, a2;
long long C;

void ComputeBits() {
	bit[0] = 1;
	for (int i = 1; i < NMAX; i++)
		bit[i] = (bit[i - 1] * 2) % MOD;
}

inline int extended_gcd(int A, int B, int &X, int &Y) {
	if (B == 0) {
		X = 1;
		Y = 0;
		return A;
	}

	int X0, Y0, D;
	D = extended_gcd(B, A % B, X0, Y0);
	X = Y0;
	Y = X0 - (A / B) * Y0;
	return D;
}

void ComputeInv() {
	int i, garbage;
	
	fact[1] = 1;
	for (i = 2; i < NMAX; i++)
		fact[i] = ((long long) fact[i - 1] * (long long) i) % MOD;

	extended_gcd(fact[NMAX - 1], MOD, factinv[NMAX - 1], garbage);
	factinv[NMAX - 1] %= MOD;
	while (factinv[NMAX - 1] < 0)
		factinv[NMAX - 1] += MOD;
	
	for (i = NMAX - 2; i >= 1; i--) {
		factinv[i] = ((long long) factinv[i + 1] * (long long) (i + 1)) % MOD;
	}

	inv[1] = 1;
	for (i = 2; i < NMAX; i++) {
		inv[i] = ((long long) fact[i - 1] * (long long) factinv[i]) % MOD;
	}
}

void Sieve() {
	int i, j;

	memset(bad, 0, sizeof(bad));
	memset(nprime, 0, sizeof(nprime));
	
	for (i = 2; i < NMAX; i++) {
		if (nprime[i] != 0)
			continue;
		nprime[i] = 1;
		for (j = 2 * i; j < NMAX; j += i) {
			nprime[j]++;
			if (j % (i * i) == 0)
				bad[j] = 1;
		}
	}
}

int q;

void ComputeGoodDivisors() {
	int i, j;
	for (i = 2; i < NMAX; i++) {
		ngooddiv[i] = 0;
		for (j = 2; j * j <= i; j++)
			if (i % j == 0) {
				if (!bad[j]) {
					ngooddiv[i]++;
					gooddiv[i][ngooddiv[i]] = j;
				}
				if (j * j != i) {
					q = i / j;
					if (!bad[q]) {
						ngooddiv[i]++;
						gooddiv[i][ngooddiv[i]] = q;
					}
				}
			}
	
		if (!bad[i]) {
			ngooddiv[i]++;
			gooddiv[i][ngooddiv[i]] = i;
		}
	}
}

void ComputeComb(int K) {
	int i;
	for (i = 0; i < K; i++)
		comb[i] = 0;
	comb[K] = 1;
	for (i = K + 1; i < NMAX; i++) {
		comb[i] = ((long long) comb[i - 1] * (long long) i) % MOD;
		comb[i] = ((long long) comb[i] * (long long) inv[i - K]) % MOD;
	}
}

int ncalls, nelems;

void Solve(int level, int prime) {
	int i, j, k;

	ncalls++;
	if (DEBUG)
		fprintf(stderr, "level=%d prime=%d base=%d vsize=%d vmax=%d vsz=%d\n", level, prime, base[level], vsize[level], vmax[level], vsz[level]);
	
	a1 = bit[vsize[level]] - 1;
	a2 = comb[vsize[level]];

	touch_idx++;
	ncnt[level] = 0;

	for (k = 1; k <= vsz[level]; k++) {
		i = vlist[level][k];
		for (j = 1; j <= ngooddiv[i]; j++) {
			q = gooddiv[i][j];
			nelems++;
			
			if (touched[level][q] != touch_idx) {
				touched[level][q] = touch_idx;
				cnt[level][q] = 0;
				ncnt[level]++;
				lcnt[level][ncnt[level]] = q;
			}
			
			cnt[level][q] += tcnt[level][i];
		}
	}

	for (k = 1; k <= ncnt[level]; k++) {
		i = lcnt[level][k];
		//if (DEBUG)
		//	fprintf(stderr, "  i=%d: cnt=%d\n", i, cnt[level][i]);

		if (nprime[i] % 2 == 1) {
			a1 = (((a1 - (bit[cnt[level][i]] - 1)) % MOD) + MOD) % MOD;
			a2 = (((a2 - comb[cnt[level][i]]) % MOD) + MOD) % MOD;
		} else {
			a1 = (((a1 + (bit[cnt[level][i]] - 1)) % MOD) + MOD) % MOD;
			a2 = (a2 + comb[cnt[level][i]]) % MOD;
		}
	}

	if (DEBUG)
		fprintf(stderr, "  num_subsets: %d %d\n", a1, a2);
	
	a1 = (((C / base[level]) % MOD) * (long long) a1) % MOD;
	a2 = (((C / base[level]) % MOD) * (long long) a2) % MOD;

	if (DEBUG)
		fprintf(stderr, "  num_subsets x factor: %d %d\n", a1, a2);
	
	answer1 = (answer1 + a1) % MOD;
	answer2 = (answer2 + a2) % MOD;
	
	for (; prime <= vmax[level]; prime++) {
		if (nprime[prime] != 1 || bad[prime])
			continue;
	
		vsize[level + 1] = vmax[level + 1] = 0;
		for (j = prime; j <= vmax[level]; j += prime) {
			vsize[level + 1] += tcnt[level][j];
			if (tcnt[level][j] > 0) {
				vmax[level + 1] = j;
				nelems++;
			}
		}

		if (vsize[level + 1] > 0) {
			//fprintf(stderr, "  %d\n", prime);
			vmax[level + 1] /= prime;
			memset(tcnt[level + 1], 0, (vmax[level + 1] + 1) * sizeof(tcnt[level + 1][0]));
			vsz[level + 1] = 0;
			for (j = prime, q = 1; j <= vmax[level]; j += prime, q++) {
				if (tcnt[level][j] > 0) {
					tcnt[level + 1][q] = tcnt[level][j];
					vsz[level + 1]++;
					vlist[level + 1][vsz[level + 1]] = q;
				}
			}

			base[level + 1] = base[level] * prime;
			Solve(level + 1, prime);
		}
	}
}

int main() {
	ComputeBits();
	ComputeInv();
	Sieve();
	ComputeGoodDivisors();

	//freopen("x.txt", "r", stdin);
	int T, te, c, i;
	scanf("%d", &T);

	for (te = 1; te <= T; te++) {
		scanf("%d %lld %d", &N, &C, &K);
		ComputeComb(K);
		
		scanf("%d %d %d", &v[1], &v[2], &c);

		for (i = 3; i <= N; i++)
			v[i] = 1 + ((long long) v[1] * (long long) v[i - 2] + (long long) v[2] * (long long) v[i - 1] + c) % 100000;

		memset(tcnt[0], 0, sizeof(tcnt[0]));
		vmax[0] = vsz[0] = 0;
		for (i = 1; i <= N; i++) {
			if (DEBUG)
				fprintf(stderr, "i=%d: v=%d\n", i, v[i]);
			tcnt[0][v[i]]++;
			if (v[i] > vmax[0])
				vmax[0] = v[i];
			if (tcnt[0][v[i]] == 1) {
				vsz[0]++;
				vlist[0][vsz[0]] = v[i];
			}
		}
		
		base[0] = 1;
		vsize[0] = N;
		ncalls = nelems = answer1 = answer2 = 0;
		Solve(0, 2);
		printf("Case %d: %d %d\n", te, answer1, answer2);
		if (DEBUG)
			fprintf(stderr, "ncalls=%d nelems=%d\n", ncalls, nelems);
	}
	
	return 0;
}
